@@ -1,5 +1,5 @@
 /*
- *  $Id: file.c,v 1.1 1998/12/02 23:37:06 ezk Exp $
+ *  $Id: file.c,v 1.2 1999/01/10 22:43:38 ezk Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,7 +48,7 @@ cryptfs_llseek(file_t *file, loff_t offset, int origin)
     if (hidden_file->f_op && hidden_file->f_op->llseek) {
 	err = hidden_file->f_op->llseek(hidden_file, offset, origin);
     } else {
-	err = generic_llseek(hidden_file, offset, origin);
+	err = default_llseek(hidden_file, offset, origin);
     }
     if (err < 0) {
 	goto out;
