@@ -1,5 +1,5 @@
 /*
- *  $Id: inode.c,v 1.1 1998/12/02 23:37:07 ezk Exp $
+ *  $Id: inode.c,v 1.2 1999/01/10 22:43:38 ezk Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -79,7 +79,7 @@ cryptfs_lookup(inode_t *dir, dentry_t *dentry)
 	fist_print_inode("LOOKUP: hidden_dir_dentry->d_inode",
 			 hidden_dir_dentry->d_inode);
 
-    /* must initialize dentryu operations */
+    /* must initialize dentry operations */
     dentry->d_op = &cryptfs_dops;
 
     /* increase refcount of base dentry (lookup_dentry will decrement) */
@@ -313,6 +313,59 @@ cryptfs_mkdir(inode_t *dir, dentry_t *dentry, int mode)
 }
 
 
+// 2.1.132
+// cryptfs_rmdir(inode_t *dir, const char *name, int namelen)
+STATIC int
+cryptfs_rmdir(inode_t *dir, dentry_t *dentry)
+{
+    int err = -EPERM;
+    dentry_t *hidden_dentry = cryptfs_hidden_dentry(dentry);
+    dentry_t *hidden_dir_dentry;
+
+    print_entry_location();
+
+    hidden_dir_dentry = dget(hidden_dentry->d_parent);
+    err = PTR_ERR(hidden_dir_dentry);
+    if (IS_ERR(hidden_dir_dentry))
+	goto out;
+
+    if (!hidden_dentry->d_inode) {
+	err = -ENOENT;
+	goto out_dput;
+    }
+
+    fist_checkinode(dir, "cryptfs_rmdir-dir");
+    if (!hidden_dir_dentry->d_inode->i_op ||
+	!hidden_dir_dentry->d_inode->i_op->rmdir) {
+	err = -EPERM;
+	goto out_dput;
+    }
+
+    /*
+     * The dentry->d_count stuff confuses d_delete() enough to
+     * not kill the inode from under us while it is locked. This
+     * wouldn't be needed, except the dentry semaphore is really
+     * in the inode, not in the dentry..
+     */
+    hidden_dentry->d_count++;
+    double_lock(hidden_dir_dentry, hidden_dentry);
+
+    err = vfs_rmdir(hidden_dir_dentry->d_inode, hidden_dentry);
+    if (!err)
+        fist_copy_attr_times(dir, hidden_dir_dentry->d_inode);
+
+    double_unlock(hidden_dentry, hidden_dir_dentry);
+
+ out_dput:
+    /* don't dput here, it will be done later in d_delete */
+    //    dput(hidden_dentry);
+ out:
+    print_exit_status(err);
+    return err;
+}
+
+#if 0
+// 2.1.129
 // cryptfs_rmdir(inode_t *dir, const char *name, int namelen)
 STATIC int
 cryptfs_rmdir(inode_t *dir, dentry_t *dentry)
@@ -346,7 +399,81 @@ cryptfs_rmdir(inode_t *dir, dentry_t *dentry)
     print_exit_status(err);
     return err;
 }
+#endif
 
+#if 0
+// 2.1.131
+// cryptfs_rmdir(inode_t *dir, const char *name, int namelen)
+STATIC int
+cryptfs_rmdir(inode_t *dir, dentry_t *dentry)
+{
+    int err = -EPERM;
+    dentry_t *hidden_dentry = cryptfs_hidden_dentry(dentry);
+    dentry_t *hidden_dir_dentry;
+
+    print_entry_location();
+
+    hidden_dir_dentry = dget(hidden_dentry->d_parent);
+    if (!hidden_dentry->d_inode) {
+	err = -ENOENT;
+	goto out;
+    }
+
+    fist_checkinode(dir, "cryptfs_rmdir-dir");
+    if (!hidden_dir_dentry->d_inode->i_op || !hidden_dir_dentry->d_inode->i_op->rmdir) {
+	err = -EPERM;
+	goto out_lock;
+    }
+
+    /*
+     * The dentry->d_count stuff confuses d_delete() enough to
+     * not kill the inode from under us while it is locked. This
+     * wouldn't be needed, except the dentry semaphore is really
+     * in the inode, not in the dentry..
+     */
+    hidden_dentry->d_count++;
+    double_lock(hidden_dir_dentry, hidden_dentry);
+    if (hidden_dentry->d_parent != hidden_dir_dentry)
+	goto out_lock;
+
+    /*
+     * We try to drop the dentry early: we should have
+     * a usage count of 2 if we're the only user of this
+     * dentry, and if that is true (possibly after pruning
+     * the dcache), then we drop the dentry now.
+     *
+     * A low-level filesystem can, if it choses, legally
+     * do a
+     *
+     *	if (!list_empty(&dentry->d_hash))
+     *		return -EBUSY;
+     *
+     * if it cannot handle the case of removing a directory
+     * that is still in use by something else..
+     */
+    switch (hidden_dentry->d_count) {
+    default:
+	shrink_dcache_parent(hidden_dentry);
+	if (hidden_dentry->d_count != 2)
+	    break;
+    case 2:
+	d_drop(hidden_dentry);
+    }
+
+    err = hidden_dir_dentry->d_inode->i_op->rmdir(hidden_dir_dentry->d_inode,
+						  hidden_dentry);
+    fist_checkinode(dir, "post cryptfs_rmdir-dir");
+    if (!err)
+        fist_copy_attr_times(dir, hidden_dir_dentry->d_inode);
+
+ out_lock:
+    hidden_dentry->d_count--;
+    double_unlock(hidden_dentry, hidden_dir_dentry);
+ out:
+    print_exit_status(err);
+    return err;
+}
+#endif
 
 // cryptfs_mknod(inode_t *dir, const char *name, int namelen, int mode, int dev)
 STATIC int
@@ -700,6 +827,7 @@ restart_page:
 	    goto out_free;
 	}
     }
+    /* XXX: what if the lower f/s doesn't implement readpage? */
 
     /*
      * decode hidden_page data onto page
