@@ -1,8 +1,14 @@
 #!/bin/bash -norc
-#set -x
+set -x
 
 TOP=/usr/local/fist
-$TOP/sbin/fist_ioctl /mnt/cryptfs ${1:-6}
+$TOP/sbin/fist_ioctl /mnt/cryptfs ${1:-18}
+
+mkdir /mnt/cryptfs/a.$$
+read n
+rmdir /mnt/cryptfs/a.$$
+
+exit
 
 cd /mnt/cryptfs/am-utils-6.0b2s3/A.i586-pc-linux-gnu/amq
 strace as -V -Qy -o amq.o amq.s 2>&1
