@@ -1,7 +1,12 @@
-#include <stdio.h>
+#include <sys/ioctl.h>
 #include <sys/fcntl.h>
+#include <unistd.h>
+#include <stdio.h>
+
+#include <fist.h>
 #include <cryptfs.h>
 
+int
 main(int argc, char *argv[])
 {
   int fd, ret;
